// Generated by CoffeeScript 1.9.1

/*
 * Composition Mod for CodeMirror
 * v5.0.0
 * mizchi <miz404@gmail.com>
 * Zhusee <zhusee2@gmail.com>
#
 * Additional instance properties added to CodeMirror:
 *   - cm.display.inCompositionMode (Boolen)
 *   - cm.display.textMarkerInComposition (TextMarker)
 */

(function() {
  var PREFIX_LIST, TEXT_MARKER_CLASS_NAME, TEXT_MARKER_OPTIONS, capitalizeString, clearCompositionTextMarkers, getPrefixedPropertyName, modInitialized, resetInputTranslate, setInputTranslate;

  modInitialized = false;

  TEXT_MARKER_CLASS_NAME = "cm-text-in-composition";

  TEXT_MARKER_OPTIONS = {
    inclusiveLeft: true,
    inclusiveRight: true,
    className: TEXT_MARKER_CLASS_NAME
  };

  PREFIX_LIST = ['webkit', 'moz', 'o'];

  capitalizeString = function(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  };

  getPrefixedPropertyName = function(propertyName) {
    var i, len, prefix, prefixedPropertyName, tempElem;
    tempElem = document.createElement('div');
    if (tempElem.style[propertyName] != null) {
      return propertyName;
    }
    for (i = 0, len = PREFIX_LIST.length; i < len; i++) {
      prefix = PREFIX_LIST[i];
      prefixedPropertyName = prefix + capitalizeString(propertyName);
      if (tempElem.style[prefixedPropertyName] != null) {
        return prefixedPropertyName;
      }
    }
    return false;
  };

  setInputTranslate = function(cm, translateValue) {
    var transformProperty;
    transformProperty = getPrefixedPropertyName("transform");
    return cm.display.input.textarea.style[transformProperty] = translateValue;
  };

  resetInputTranslate = function(cm) {
    return setInputTranslate("");
  };

  clearCompositionTextMarkers = function(cm) {
    var i, len, textMarker, textMarkersArray;
    textMarkersArray = cm.getAllMarks();
    for (i = 0, len = textMarkersArray.length; i < len; i++) {
      textMarker = textMarkersArray[i];
      if ((textMarker != null) && textMarker.className === TEXT_MARKER_CLASS_NAME) {
        textMarker.clear();
      }
    }
    return true;
  };

  module.exports = function(CodeMirror) {
    var initCompositionMode;
    CodeMirror.defineOption('enableCompositionMod', false, function(cm, newVal, oldVal) {
      if (newVal && !modInitialized) {
        if (window.CompositionEvent != null) {
          return initCompositionMode(cm);
        } else {
          console.warn("Your browser doesn't support CompositionEvent.");
          return cm.setOption('enableCompositionMod', false);
        }
      }
    });
    return initCompositionMode = function(cm) {
      var cmWrapper, inputField, inputWrapper;
      inputField = cm.display.input.textarea;
      inputWrapper = cm.display.input.textarea.parentElement;
      cmWrapper = cm.display.wrapper;
      inputWrapper.classList.add('CodeMirror-input-wrapper');
      CodeMirror.on(inputField, 'compositionstart', function(event) {
        if (!cm.options.enableCompositionMod) {
          return;
        }
        cm.display.inCompositionMode = true;
        cm.setOption('readOnly', true);
        if (cm.somethingSelected()) {
          cm.replaceSelection("");
        }
        cm.display.compositionHead = cm.getCursor();
        inputField.value = "";
        inputWrapper.classList.add('in-composition');
        return cmWrapper.classList.add('cm-in-composition');
      });
      CodeMirror.on(inputField, 'compositionupdate', function(event) {
        var endPos, headPos, markerRange, pixelToTranslate;
        if (!cm.options.enableCompositionMod) {
          return;
        }
        headPos = cm.display.compositionHead;
        if (cm.display.textMarkerInComposition) {
          markerRange = cm.display.textMarkerInComposition.find();
          cm.replaceRange(event.data, headPos, markerRange.to);
          cm.display.textMarkerInComposition.clear();
          cm.display.textMarkerInComposition = void 0;
        } else {
          cm.replaceRange(event.data, headPos, headPos);
        }
        endPos = cm.getCursor();
        cm.display.textMarkerInComposition = cm.markText(headPos, endPos, TEXT_MARKER_OPTIONS);
        pixelToTranslate = cm.charCoords(endPos).left - cm.charCoords(headPos).left;
        return setInputTranslate(cm, "translateX(-" + pixelToTranslate + "px)");
      });
      return CodeMirror.on(inputField, 'compositionend', function(event) {
        var endPos, headPos, postCompositionEnd, ref, textLeftComposition;
        if (!cm.options.enableCompositionMod) {
          return;
        }
        cmWrapper.classList.remove('cm-in-composition');
        textLeftComposition = event.data;
        headPos = cm.display.compositionHead;
        endPos = cm.getCursor();
        cm.display.inCompositionMode = false;
        cm.display.compositionHead = void 0;
        if ((ref = cm.display.textMarkerInComposition) != null) {
          ref.clear();
        }
        cm.display.textMarkerInComposition = void 0;
        cm.setOption('readOnly', false);
        inputWrapper.classList.remove('in-composition');
        clearCompositionTextMarkers(cm);
        postCompositionEnd = function() {
          if (cm.display.inCompositionMode) {
            return false;
          }
          inputField.value = "";
          return CodeMirror.off(inputField, 'input', postCompositionEnd);
        };
        return CodeMirror.on(inputField, 'input', postCompositionEnd);
      });
    };
  };

}).call(this);
