###
# Composition Mod for CodeMirror
# v5.0.0
# mizchi <miz404@gmail.com>
# Zhusee <zhusee2@gmail.com>
#
# Additional instance properties added to CodeMirror:
#   - cm.display.inCompositionMode (Boolen)
#   - cm.display.textMarkerInComposition (TextMarker)
###

modInitialized = false
TEXT_MARKER_CLASS_NAME = "cm-text-in-composition"
TEXT_MARKER_OPTIONS = {
  inclusiveLeft: true,
  inclusiveRight: true,
  className: TEXT_MARKER_CLASS_NAME
}
PREFIX_LIST = ['webkit', 'moz', 'o']

capitalizeString = (string) ->
  return string.charAt(0).toUpperCase() + string.slice(1)

getPrefixedPropertyName = (propertyName) ->
  tempElem = document.createElement('div')

  return propertyName if tempElem.style[propertyName]?

  for prefix in PREFIX_LIST
    prefixedPropertyName = prefix + capitalizeString(propertyName)
    return prefixedPropertyName if tempElem.style[prefixedPropertyName]?

  return false

setInputTranslate = (cm, translateValue) ->
  transformProperty = getPrefixedPropertyName("transform")
  cm.display.input.textarea.style[transformProperty] = translateValue

resetInputTranslate = (cm) ->
  setInputTranslate("")

clearCompositionTextMarkers = (cm)->
  # Clear previous text markers
  textMarkersArray = cm.getAllMarks()
  for textMarker in textMarkersArray
    if textMarker? and textMarker.className is TEXT_MARKER_CLASS_NAME
      textMarker.clear()

  return true

module.exports = (CodeMirror) ->
  CodeMirror.defineOption 'enableCompositionMod', false, (cm, newVal, oldVal) ->
    if newVal and !modInitialized
      if window.CompositionEvent?
        initCompositionMode(cm)
      else
        console.warn("Your browser doesn't support CompositionEvent.")
        cm.setOption('enableCompositionMod', false)

  initCompositionMode = (cm) ->
    inputField = cm.display.input.textarea
    inputWrapper = cm.display.input.textarea.parentElement
    cmWrapper = cm.display.wrapper

    inputWrapper.classList.add('CodeMirror-input-wrapper')
    CodeMirror.on inputField, 'compositionstart', (event) ->
      return if !cm.options.enableCompositionMod

      cm.display.inCompositionMode = true
      cm.setOption('readOnly', true)

      cm.replaceSelection("") if cm.somethingSelected() # Clear the selected text first

      cm.display.compositionHead = cm.getCursor()

      inputField.value = ""

      inputWrapper.classList.add('in-composition')
      cmWrapper.classList.add('cm-in-composition')

    CodeMirror.on inputField, 'compositionupdate', (event) ->
      return if !cm.options.enableCompositionMod
      headPos = cm.display.compositionHead

      if cm.display.textMarkerInComposition
        markerRange = cm.display.textMarkerInComposition.find()
        cm.replaceRange(event.data, headPos, markerRange.to)
        cm.display.textMarkerInComposition.clear()
        cm.display.textMarkerInComposition = undefined
      else
        cm.replaceRange(event.data, headPos, headPos)

      endPos = cm.getCursor()
      cm.display.textMarkerInComposition = cm.markText(headPos, endPos, TEXT_MARKER_OPTIONS)

      pixelToTranslate = cm.charCoords(endPos).left - cm.charCoords(headPos).left
      setInputTranslate(cm, "translateX(-#{pixelToTranslate}px)")

    CodeMirror.on inputField, 'compositionend', (event) ->
      return if !cm.options.enableCompositionMod
      cmWrapper.classList.remove('cm-in-composition')

      textLeftComposition = event.data
      headPos = cm.display.compositionHead
      endPos = cm.getCursor()

      cm.display.inCompositionMode = false
      cm.display.compositionHead = undefined
      cm.display.textMarkerInComposition?.clear()
      cm.display.textMarkerInComposition = undefined
      cm.setOption('readOnly', false)

      inputWrapper.classList.remove('in-composition')

      clearCompositionTextMarkers(cm)

      postCompositionEnd = ->
        return false if cm.display.inCompositionMode
        inputField.value = ""
        CodeMirror.off(inputField, 'input', postCompositionEnd)

      CodeMirror.on(inputField, 'input', postCompositionEnd)
